#include <sigc++/bind.h>
#include <msp/fs/dir.h>
#include <msp/graphics/display_private.h>
#include <msp/time/utils.h>
#include "xineengine.h"
#include "xinema.h"
#include "xinestream.h"

using namespace std;
using namespace Msp;

Xinema::EarlyInit::EarlyInit()
{
	XInitThreads();
}


Xinema::Xinema(int, char **):
	window(display, 1920, 1080),
	network(*this),
	engine(0),
	stream(0),
	logo_mode(false),
	pending_logo(false)
{
	window.signal_close.connect(sigc::bind(sigc::mem_fun(this, &Xinema::exit), 0));
}

int Xinema::main()
{
	window.show();
	display.tick();

	engine = new XineEngine(window);
	show_logo();

	Application::main();

	if(stream)
		delete stream;
	delete engine;

	return exit_code;
}

void Xinema::tick()
{
	string mrl;
	bool logo;
	{
		MutexLock lock(command_mutex);
		mrl = pending_mrl;
		logo = pending_logo;
		pending_mrl.clear();
		pending_logo = false;
	}

	if(!mrl.empty())
	{
		bool had_stream = stream;

		delete stream;
		stream = 0;

		if(had_stream && !logo_mode)
			signal_stream_destroyed.emit();

		logo_mode = logo;
		stream = new XineStream(*engine, mrl);
		if(!logo_mode)
			stream->signal_finished.connect(sigc::mem_fun(this, &Xinema::show_logo));
		stream->play();

		if(!logo_mode)
			signal_stream_created.emit(*stream);
	}

	XLockDisplay(display.get_private().display);
	display.tick();
	XUnlockDisplay(display.get_private().display);

	engine->tick();

	Time::sleep(10*Time::msec);
}

void Xinema::play_file(const FS::Path &fn)
{
	MutexLock lock(command_mutex);
	pending_mrl = "file://"+fn.str();
	pending_logo = false;
}

void Xinema::show_logo()
{
	MutexLock lock(command_mutex);
	pending_mrl = "file://"+(FS::get_sys_data_dir()/"xinema.png").str();
	pending_logo = true;
}
